// evive TFT Display.js

(function(ext) {
    var device = null;
    
    var levels = {
        HIGH:1,
        LOW:0
    };

    var TFTColor = {
        "Black": 0,
        "Navy": 15,
        "Dark Green": 992,
        "Dark Cyan": 1007,
        "Maroon": 30720,
        "Purple": 30735,
        "Olive": 31712,
        "Light Grey": 50712,
        "Dark Grey": 31727,
        "Blue": 31,
        "Green": 2016,
        "Cyan": 2047,
        "Red": 63488,
        "Magneta": 63519,
        "Yellow": 65504,
        "White": 65535,
        "Orange": 64800,
        "Pink": 63519,
    };
    
    var tab = {
        GREENTAB:0,
        BLACKTAB:2,
    };

    

    ext.resetAll = function(){};

    ext.printText = function(message){
        runPackage(13,string2array(message));
    };

    ext.initTFT = function(color) {
        runPackage(61, typeof color=="string"?tab[color]:new Number(color));
    };

    ext.fillScreen = function(color){
        runPackage(14,short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.setCursor = function(posX, posY){
        runPackage(15, posX, posY);
    };

    ext.setTextColor = function(color1, color2){
        runPackage(16,0,short2array(typeof color1=="number"?color1:TFTColor[color1]), short2array(typeof color2=="number"?color2:TFTColor[color2]));
    };

    ext.setTextSize = function(textSize){
        runPackage(17, textSize);
    };

    ext.drawLine = function(x1, y1, x2, y2, color){
        runPackage(18, x1, y1, x2, y2, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawVLine = function(x1, y1, l, color){
        runPackage(19, x1, y1, l, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawHLine = function(x1, y1, l, color){
        runPackage(20, x1, y1, l, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.fillRectangle = function(x1, y1, w, h, color){
        runPackage(21, x1, y1, w, h, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.fillRoundRectangle = function(x1, y1, w, h, r, color){
        runPackage(22, x1, y1, w, h, r, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.fillCircle = function(x1, y1, r, color){
        runPackage(23, x1, y1, r, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.fillEllipse = function(x1, y1, rx, ry, color){
        runPackage(24, x1, y1, rx, ry, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.fillTriangle = function(x1, y1, x2, y2, x3, y3, color){
        runPackage(25, x1, y1, x2, y2, x3, y3, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawRectangle = function(x1, y1, w, h, color){
        runPackage(26, x1, y1, w, h, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawRoundRectangle = function(x1, y1, w, h, r, color){
        runPackage(27, x1, y1, w, h, r, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawCircle = function(x1, y1, r, color){
        runPackage(28, x1, y1, r, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawEllipse = function(x1, y1, rx, ry, color){
        runPackage(29, x1, y1, rx, ry, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.drawTriangle = function(x1, y1, x2, y2, x3, y3, color){
        runPackage(60, x1, y1, x2, y2, x3, y3, short2array(typeof color=="number"?color:TFTColor[color]));
    };

    ext.runArduino = function(){
        
    };

    function sendPackage(argList, type){
        var bytes = [0xff, 0x55, 0, 0, type];
        for(var i=0;i<argList.length;++i){
            var val = argList[i];
            if(val.constructor == "[class Array]"){
                bytes = bytes.concat(val);
            }else{
                bytes.push(val);
            }
        }
        bytes[2] = bytes.length - 3;
        device.send(bytes);
    }
    
    function runPackage(){
        sendPackage(arguments, 2);
    }
    function getPackage(){
        var nextID = arguments[0];
        Array.prototype.shift.call(arguments);
        sendPackage(arguments, 1);
    }

    var inputArray = [];
    var _isParseStart = false;
    var _isParseStartIndex = 0;
    function processData(bytes) {
        var len = bytes.length;
        if(_rxBuf.length>30){
            _rxBuf = [];
        }
        for(var index=0;index<bytes.length;index++){
            var c = bytes[index];
            _rxBuf.push(c);
            if(_rxBuf.length>=2){
                if(_rxBuf[_rxBuf.length-1]==0x55 && _rxBuf[_rxBuf.length-2]==0xff){
                    _isParseStart = true;
                    _isParseStartIndex = _rxBuf.length-2;
                }
                if(_rxBuf[_rxBuf.length-1]==0xa && _rxBuf[_rxBuf.length-2]==0xd&&_isParseStart){
                    _isParseStart = false;
                    
                    var position = _isParseStartIndex+2;
                    var extId = _rxBuf[position];
                    position++;
                    var type = _rxBuf[position];
                    position++;
                    //1 byte 2 float 3 short 4 len+string 5 double
                    var value;
                    switch(type){
                        case 1:{
                            value = _rxBuf[position];
                            position++;
                        }
                            break;
                        case 2:{
                            value = readFloat(_rxBuf,position);
                            position+=4;
                        }
                            break;
                        case 3:{
                            value = readInt(_rxBuf,position,2);
                            position+=2;
                        }
                            break;
                        case 4:{
                            var l = _rxBuf[position];
                            position++;
                            value = readString(_rxBuf,position,l);
                        }
                            break;
                        case 5:{
                            value = readDouble(_rxBuf,position);
                            position+=4;
                        }
                            break;
                        case 6:
                            value = readInt(_rxBuf,position,4);
                            position+=4;
                            break;
                    }
                    if(type<=6){
                        if (responsePreprocessor[extId] && responsePreprocessor[extId] != null) {
                            value = responsePreprocessor[extId](value);
                            responsePreprocessor[extId] = null;
                        }
                        responseValue(extId,value);
                    }else{
                        responseValue();
                    }
                    _rxBuf = [];
                }
            } 
        }
    }
    function readFloat(arr,position){
        var f= [arr[position],arr[position+1],arr[position+2],arr[position+3]];
        return parseFloat(f);
    }
    function readInt(arr,position,count){
        var result = 0;
        for(var i=0; i<count; ++i){
            result |= arr[position+i] << (i << 3);
        }
        return result;
    }
    function readDouble(arr,position){
        return readFloat(arr,position);
    }
    function readString(arr,position,len){
        var value = "";
        for(var ii=0;ii<len;ii++){
            value += String.fromCharCode(_rxBuf[ii+position]);
        }
        return value;
    }
    function appendBuffer( buffer1, buffer2 ) {
        return buffer1.concat( buffer2 );
    }

    // Extension API interactions
    var potentialDevices = [];
    ext._deviceConnected = function(dev) {
        potentialDevices.push(dev);
        if (!device) {
            tryNextDevice();
        }
    }

    function tryNextDevice() {
        // If potentialDevices is empty, device will be undefined.
        // That will get us back here next time a device is connected.
        device = potentialDevices.shift();
        if (device) {
            device.open({ stopBits: 0, bitRate: 115200, ctsFlowControl: 0 }, deviceOpened);
        }
    }

    var watchdog = null;
    function deviceOpened(dev) {
        if (!dev) {
            // Opening the port failed.
            tryNextDevice();
            return;
        }
        device.set_receive_handler('makeblock',processData);
    };

    ext._deviceRemoved = function(dev) {
        if(device != dev) return;
        device = null;
    };

    ext._shutdown = function() {
        if(device) device.close();
        device = null;
    };

    ext._getStatus = function() {
        if(!device) return {status: 1, msg: 'Arduino disconnected'};
        if(watchdog) return {status: 1, msg: 'Probing for Arduino'};
        return {status: 2, msg: 'Arduino connected'};
    }

    var descriptor = {};
    ScratchExtensions.register('Arduino', descriptor, ext, {type: 'serial'});
})({});
